static const char *RcsId = "$Id: DevServ.cpp 13293 2009-04-07 10:53:56Z pascal_verdier $";
//+=============================================================================
//
// file :         MIO.cpp
//
// description :  C++ source for the MIO and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                MIO are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: pascal_verdier $
//
// $Revision: 13293 $
//
// $Revision: 13293 $
// $Date: 2009-04-07 12:53:56 +0200 (Tue, 07 Apr 2009) $
//
// SVN only:
// $HeadURL: $
//
// CVS only:
// $Source$
// $Log$
// Revision 3.5  2007/10/24 12:07:35  pascal_verdier
// Another spelling mistake correction
//
// Revision 3.4  2007/10/23 14:04:30  pascal_verdier
// Spelling mistakes correction
//
// Revision 3.3  2005/03/02 14:06:15  pascal_verdier
// namespace is different than class name.
//
// Revision 3.2  2004/11/08 11:33:16  pascal_verdier
// if device property not found in database, it takes class property value if exists.
//
// Revision 3.1  2004/09/06 09:27:05  pascal_verdier
// Modified for Tango 5 compatibility.
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name      |  Method name
//	----------------------------------------
//  State             |  dev_state()
//  Status            |  dev_status()
//  write_digit       |  write_digit()
//  write_freq        |  write_freq()
//  write_conf        |  write_conf()
//  write_irqtoutena  |  write_irqtoutena()
//  write_irqtout     |  write_irqtout()
//  write_pulse       |  write_pulse()
//  read_digit        |  read_digit()
//  read_digits       |  read_digits()
//  read_freq         |  read_freq()
//  read_conf         |  read_conf()
//  read_digitset     |  read_digitset()
//  read_digitsets    |  read_digitsets()
//  read_freqset      |  read_freqset()
//  read_irqtoutstat  |  read_irqtoutstat()
//  read_irqtout      |  read_irqtout()
//
//===================================================================


#include <tango.h>
#include <MIO.h>
#include <MIOClass.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>


namespace MIO_ns
{

//+----------------------------------------------------------------------------
//
// method : 		MIO::MIO(string &s)
// 
// description : 	constructor for simulated MIO
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
MIO::MIO(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

MIO::MIO(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

MIO::MIO(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
/*
 * timestamp() 
 * send a timestamp to standard output
 */
void MIO::timestamp() 
{
	struct tm *mytime;
	char timeLong[80];
	time_t myTime_t;
	myTime_t = time(NULL);
	mytime = localtime(&myTime_t);
	strftime(timeLong, 79, "%Y-%m-%d %H:%M:%S ", mytime);
	cout << timeLong;
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void MIO::delete_device()
{
	//	Delete device's allocated object
	close(mio_dev);
	
	delete attr_Freq_chan0_read;
	delete attr_Freq_chan1_read;		
	delete attr_Freq_chan2_read;
	delete attr_Freq_chan3_read;
	delete attr_Freq_chan4_read;		
	delete attr_Freq_chan5_read;	
	delete attr_Freq_chan6_read;
	delete attr_Freq_chan7_read;		
	delete attr_Freq_chan8_read;
	delete attr_Freq_chan9_read;
	delete attr_Freq_chan10_read;		
	delete attr_Freq_chan11_read;	
	delete attr_Freq_chan12_read;
	delete attr_Freq_chan13_read;
	delete attr_Freq_chan14_read;		
	delete attr_Freq_chan15_read;						
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void MIO::init_device()
{
	INFO_STREAM << "MIO::MIO() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	mio_t mio_val;

	//	Add your own code to control device here
	int err;
	// open driver, e.g. mio_name = "/dev/mio0"
	if ((mio_dev = open(mio_name.c_str(), O_RDWR)) < 0) {
		set_state(Tango::DISABLE);
		set_status("Failed to open Multi I/O driver");
		cout << "cannot open driver: " << mio_name.c_str() << " \n" << endl;
		return;
	}
	set_state(Tango::ON);
	set_status("Multi I/O enabled");
	// init chnnels
	for (int i=0; i<16; i++) {
		mio_val.chan = i;
		mio_val.value = chan_confs[i];
		cout << i << " : " << chan_confs[i] << endl;
		if (ioctl(mio_dev, MIO_WRCONF, &mio_val) < 0) {
			set_state(Tango::FAULT);
			set_status("Failed to configure Multi I/O driver");
			cout << "cannot config chan: " << i << endl;
		}				
	}
	// set trigger timeout
	mio_val.value = irq_tout;
	if (ioctl(mio_dev, MIO_WRIRQTOUT, &mio_val) < 0) {
		set_state(Tango::FAULT);
		set_status("Failed to configure trigger timeout on Multi I/O driver");
		cout << "Failed to configure trigger timeout on Multi I/O driver" << endl;
	}	
	// enable trigger check			
	mio_val.value = irq_enab? 1: 0;
	cout << "irq_enable: " << irq_enab << ", val: " << mio_val.value << endl;
	if (ioctl(mio_dev, MIO_WRIRQTOUTENA, &mio_val) < 0) {
		set_state(Tango::FAULT);
		set_status("Failed to configure trigger timeout on Multi I/O driver");
		cout << "Failed to configure trigger timeout on Multi I/O driver" << endl;
	}
	
	attr_Freq_chan0_read = new Tango::DevLong;
	attr_Freq_chan1_read = new Tango::DevLong;		
	attr_Freq_chan2_read = new Tango::DevLong;
	attr_Freq_chan3_read = new Tango::DevLong;
	attr_Freq_chan4_read = new Tango::DevLong;		
	attr_Freq_chan5_read = new Tango::DevLong;	
	attr_Freq_chan6_read = new Tango::DevLong;
	attr_Freq_chan7_read = new Tango::DevLong;		
	attr_Freq_chan8_read = new Tango::DevLong;
	attr_Freq_chan9_read = new Tango::DevLong;
	attr_Freq_chan10_read = new Tango::DevLong;		
	attr_Freq_chan11_read = new Tango::DevLong;	
	attr_Freq_chan12_read = new Tango::DevLong;
	attr_Freq_chan13_read = new Tango::DevLong;
	attr_Freq_chan14_read = new Tango::DevLong;		
	attr_Freq_chan15_read = new Tango::DevLong;						
}


//+----------------------------------------------------------------------------
//
// method : 		MIO::readDeviceProperies()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void MIO::get_device_property()
{
	//	Initialize your default values here.
	//------------------------------------------


	//	Read device properties from database.(Automatic code generation)
	//-------------------------------------------------------------
	if (Tango::Util::instance()->_UseDb==false)
		return;
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Mio_name"));
	dev_prop.push_back(Tango::DbDatum("Chan_confs"));
	dev_prop.push_back(Tango::DbDatum("Irq_tout"));
	dev_prop.push_back(Tango::DbDatum("Irq_enab"));

	//	Call database and extract values
	//--------------------------------------------
	get_db_device()->get_property(dev_prop);
	MIOClass	*ds_class =
		(static_cast<MIOClass *>(get_device_class()));
	int	i = -1;
	//	Extract Mio_name value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  mio_name;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  mio_name;
	}

	//	Extract Chan_confs value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  chan_confs;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  chan_confs;
	}

	//	Extract Irq_tout value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  irq_tout;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  irq_tout;
	}

	//	Extract Irq_enab value
	if (dev_prop[++i].is_empty()==false)	dev_prop[i]  >>  irq_enab;
	else
	{
		//	Try to get value from class property
		Tango::DbDatum	cl_prop = ds_class->get_class_property(dev_prop[i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  irq_enab;
	}



	//	End of Automatic code generation
	//-------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		MIO::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void MIO::always_executed_hook()
{
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void MIO::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "MIO::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan0
// 
// description : 	Extract real attribute values for Freq_chan0 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan0(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan0_read = read_chan_set_frequency(0);
	attr.set_value(attr_Freq_chan0_read);

}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan0
// 
// description : 	Write Freq_chan0 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan0(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan0_write);
	write_chan_frequency(0, attr_Freq_chan0_write);
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan1
// 
// description : 	Extract real attribute values for Freq_chan1 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan1(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan1_read = read_chan_set_frequency(1);
	attr.set_value(attr_Freq_chan1_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan1
// 
// description : 	Write Freq_chan1 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan1(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan1_write);
	write_chan_frequency(1, attr_Freq_chan1_write);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan2
// 
// description : 	Extract real attribute values for Freq_chan2 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan2(Tango::Attribute &attr) entering... "<< endl;

	*attr_Freq_chan2_read = read_chan_set_frequency(2);
	attr.set_value(attr_Freq_chan2_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan2
// 
// description : 	Write Freq_chan2 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan2(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan2(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan2_write);
	write_chan_frequency(2, attr_Freq_chan2_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan3
// 
// description : 	Extract real attribute values for Freq_chan3 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan3(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan3(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan3_read = read_chan_set_frequency(3);
	attr.set_value(attr_Freq_chan3_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan3
// 
// description : 	Write Freq_chan3 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan3(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan3(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan3_write);
	write_chan_frequency(3, attr_Freq_chan3_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan4
// 
// description : 	Extract real attribute values for Freq_chan4 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan4(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan4(Tango::Attribute &attr) entering... "<< endl;

	*attr_Freq_chan4_read = read_chan_set_frequency(4);
	attr.set_value(attr_Freq_chan4_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan4
// 
// description : 	Write Freq_chan4 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan4(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan4(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan4_write);
	write_chan_frequency(4, attr_Freq_chan4_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan5
// 
// description : 	Extract real attribute values for Freq_chan5 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan5(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan5(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan5_read = read_chan_set_frequency(5);
	attr.set_value(attr_Freq_chan5_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan5
// 
// description : 	Write Freq_chan5 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan5(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan5(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan5_write);
	write_chan_frequency(5, attr_Freq_chan5_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan6
// 
// description : 	Extract real attribute values for Freq_chan6 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan6(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan6(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan6_read = read_chan_set_frequency(6);
	attr.set_value(attr_Freq_chan6_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan6
// 
// description : 	Write Freq_chan6 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan6(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan6(Tango::WAttribute &attr) entering... "<< endl;

	attr.get_write_value(attr_Freq_chan6_write);
	write_chan_frequency(6, attr_Freq_chan6_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan7
// 
// description : 	Extract real attribute values for Freq_chan7 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan7(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan7(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan7_read = read_chan_set_frequency(7);
	attr.set_value(attr_Freq_chan7_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan7
// 
// description : 	Write Freq_chan7 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan7(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan7(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan7_write);
	write_chan_frequency(7, attr_Freq_chan7_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan8
// 
// description : 	Extract real attribute values for Freq_chan8 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan8(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan8(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan8_read = read_chan_set_frequency(8);
	attr.set_value(attr_Freq_chan8_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan8
// 
// description : 	Write Freq_chan8 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan8(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan8(Tango::WAttribute &attr) entering... "<< endl;

	attr.get_write_value(attr_Freq_chan8_write);
	write_chan_frequency(8, attr_Freq_chan8_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan9
// 
// description : 	Extract real attribute values for Freq_chan9 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan9(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan9(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan9_read = read_chan_set_frequency(9);
	attr.set_value(attr_Freq_chan9_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan9
// 
// description : 	Write Freq_chan9 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan9(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan9(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan9_write);
	write_chan_frequency(9, attr_Freq_chan9_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan10
// 
// description : 	Extract real attribute values for Freq_chan10 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan10(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan10(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan10_read = read_chan_set_frequency(10);
	attr.set_value(attr_Freq_chan10_read);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan10
// 
// description : 	Write Freq_chan10 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan10(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan10(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan10_write);
	write_chan_frequency(10, attr_Freq_chan10_write);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan11
// 
// description : 	Extract real attribute values for Freq_chan11 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan11(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan11(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan11_read = read_chan_set_frequency(11);
	attr.set_value(attr_Freq_chan11_read);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan11
// 
// description : 	Write Freq_chan11 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan11(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan11(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan11_write);
	write_chan_frequency(11, attr_Freq_chan11_write);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan12
// 
// description : 	Extract real attribute values for Freq_chan12 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan12(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan12(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan12_read = read_chan_set_frequency(12);
	attr.set_value(attr_Freq_chan12_read);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan12
// 
// description : 	Write Freq_chan12 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan12(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan12(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan12_write);
	write_chan_frequency(12, attr_Freq_chan12_write);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan13
// 
// description : 	Extract real attribute values for Freq_chan13 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan13(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan13(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan13_read = read_chan_set_frequency(13);
	attr.set_value(attr_Freq_chan13_read);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan13
// 
// description : 	Write Freq_chan13 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan13(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan13(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan13_write);
	write_chan_frequency(13, attr_Freq_chan13_write);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan14
// 
// description : 	Extract real attribute values for Freq_chan14 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan14(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan14(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan14_read = read_chan_set_frequency(14);
	attr.set_value(attr_Freq_chan14_read);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan14
// 
// description : 	Write Freq_chan14 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan14(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan14(Tango::WAttribute &attr) entering... "<< endl;

	attr.get_write_value(attr_Freq_chan14_write);
	write_chan_frequency(14, attr_Freq_chan14_write);	
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::read_Freq_chan15
// 
// description : 	Extract real attribute values for Freq_chan15 acquisition result.
//
//-----------------------------------------------------------------------------
void MIO::read_Freq_chan15(Tango::Attribute &attr)
{
	DEBUG_STREAM << "MIO::read_Freq_chan15(Tango::Attribute &attr) entering... "<< endl;
	
	*attr_Freq_chan15_read = read_chan_set_frequency(15);
	attr.set_value(attr_Freq_chan15_read);		
	
}

//+----------------------------------------------------------------------------
//
// method : 		MIO::write_Freq_chan15
// 
// description : 	Write Freq_chan15 attribute values to hardware.
//
//-----------------------------------------------------------------------------
void MIO::write_Freq_chan15(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "MIO::write_Freq_chan15(Tango::WAttribute &attr) entering... "<< endl;
	
	attr.get_write_value(attr_Freq_chan15_write);
	write_chan_frequency(15, attr_Freq_chan15_write);	
	
}



//+------------------------------------------------------------------
/**
 *	method:	MIO::write_digit
 *
 *	description:	method to execute "write_digit"
 *	write to a digital channel
 *	chan = [0-15]
 *	value = [0|1]
 *
 * @param	argin	args0=chan args1=value
 *
 */
//+------------------------------------------------------------------
void MIO::write_digit(const Tango::DevVarUShortArray *argin)
{
	DEBUG_STREAM << "MIO::write_digit(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = (*argin)[0];
	val.value = (*argin)[1];
	// cout << "level command, chan: " << val.chan << ", val: " << val.value << ", driver: " << device_name << endl;
	if (err = ioctl(mio_dev, MIO_WRDIGIT, &val)) {
		// cout << "err, chan: " << val.chan << ", val: " << val.value << ", driver: " << device_name << endl;
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_digit() error", Tango::ERR);
	}				
	// timestamp(); cout << "write_digit(), chan: " << val.chan << ", value: " << val.value << ", driver: " << device_name << endl;

}

//+------------------------------------------------------------------
/**
 *	method:	MIO::write_freq
 *
 *	description:	method to execute "write_freq"
 *	write frequency output
 *
 * @param	argin	args0=chan args1=value
 *
 */
//+------------------------------------------------------------------
void MIO::write_freq(const Tango::DevVarUShortArray *argin)
{
	DEBUG_STREAM << "MIO::write_freq(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = (*argin)[0];
	val.value = (*argin)[1];
	if (err = ioctl(mio_dev, MIO_WRFREQ, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_freq() error", Tango::ERR);
	}				
	// timestamp(); cout << "write_freq(), chan: " << val.chan << ", value: " << val.value << ", driver: " << device_name << endl;


}

//+------------------------------------------------------------------
/**
 *	method:	MIO::write_conf
 *
 *	description:	method to execute "write_conf"
 *	Write configuration
 *	chan = [0-15]
 *	value:
 *	0 = DI
 *	1 = FI
 *	2 = DO
 *	3 = FO
 *
 * @param	argin	args0=chan args1=value
 *
 */
//+------------------------------------------------------------------
void MIO::write_conf(const Tango::DevVarUShortArray *argin)
{
	DEBUG_STREAM << "MIO::write_conf(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = (*argin)[0];
	val.value = (*argin)[1];
	if (err = ioctl(mio_dev, MIO_WRCONF, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_conf() error", Tango::ERR);
	}				


}

//+------------------------------------------------------------------
/**
 *	method:	MIO::write_irqtoutena
 *
 *	description:	method to execute "write_irqtoutena"
 *	enable timeout check on interrupt (validate data only if trigger has come recently
 *
 * @param	argin	args1=value
 *
 */
//+------------------------------------------------------------------
void MIO::write_irqtoutena(Tango::DevUShort argin)
{
	DEBUG_STREAM << "MIO::write_irqtoutena(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.value = argin;
	if (err = ioctl(mio_dev, MIO_WRIRQTOUTENA, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_irqtoutena() error", Tango::ERR);
	}				

}

//+------------------------------------------------------------------
/**
 *	method:	MIO::write_irqtout
 *
 *	description:	method to execute "write_irqtout"
 *	set timeout [s] for check of trigger presence
 *
 * @param	argin	timeout [s]
 *
 */
//+------------------------------------------------------------------
void MIO::write_irqtout(Tango::DevUShort argin)
{
	DEBUG_STREAM << "MIO::write_irqtout(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.value = argin;
	if (err = ioctl(mio_dev, MIO_WRIRQTOUT, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_irqtout() error", Tango::ERR);
	}				

}

//+------------------------------------------------------------------
/**
 *	method:	MIO::write_pulse
 *
 *	description:	method to execute "write_pulse"
 *	write a pulse
 *
 * @param	argin	args0=chan 
 *
 */
//+------------------------------------------------------------------
void MIO::write_pulse(Tango::DevUShort argin)
{
	DEBUG_STREAM << "MIO::write_pulse(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	set_state(Tango::ON);
	
	mio_t val;
	
	val.chan = argin;
	timestamp(); cout << "write_pulse(), chan: " << val.chan << ", driver: " << device_name << endl;
	if (err = ioctl(mio_dev, MIO_WRPULSE, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		cout << mio_get_errmsg(err) << endl;
		// out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_pulse() error", Tango::ERR);
	}				

}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_digit
 *
 *	description:	method to execute "read_digit"
 *	read a digital channel
 *
 * @param	argin	chan
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_digit(Tango::DevUShort argin)
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_digit(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = argin;
	if (err = ioctl(mio_dev, MIO_RDDIGIT, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << "errno:" << err << ", " << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_digit() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_digits
 *
 *	description:	method to execute "read_digits"
 *	read all digital channels
 *
 * @return	values
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_digits()
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_digits(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	if (err = ioctl(mio_dev, MIO_RDDIGITS, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_digits() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_freq
 *
 *	description:	method to execute "read_freq"
 *	read frequency value
 *
 * @param	argin	chan
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_freq(Tango::DevUShort argin)
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_freq(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = argin;
	if (err = ioctl(mio_dev, MIO_RDFREQ, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read frequency register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_freq() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_conf
 *
 *	description:	method to execute "read_conf"
 *	read configuration
 *	chan = [0-15]
 *	value:
 *	0 = DI
 *	1 = FI
 *	2 = DO
 *	3 = FO
 *
 * @param	argin	chan
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_conf(Tango::DevUShort argin)
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_conf(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = argin;
	if (err = ioctl(mio_dev, MIO_RDCONF, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_conf() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_digitset
 *
 *	description:	method to execute "read_digitset"
 *	read digital output
 *
 * @param	argin	chan
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_digitset(Tango::DevUShort argin)
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_digitset(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = argin;
	if (err = ioctl(mio_dev, MIO_RDDIGITSET, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_digitset() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_digitsets
 *
 *	description:	method to execute "read_digitsets"
 *	read all digital outputs
 *
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_digitsets()
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_digitsets(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	if (err = ioctl(mio_dev, MIO_RDDIGITSETS, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_digitsets() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_freqset
 *
 *	description:	method to execute "read_freqset"
 *	read output frequency
 *
 * @param	argin	chan
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_freqset(Tango::DevUShort argin)
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_freqset(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = argin;
	if (err = ioctl(mio_dev, MIO_RDFREQSET, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_freqset() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_irqtoutstat
 *
 *	description:	method to execute "read_irqtoutstat"
 *	read trigger timeout
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_irqtoutstat()
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_irqtoutstat(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	if (err = ioctl(mio_dev, MIO_RDIRQTOUTSTAT, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_irqtoutstat() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_irqtout
 *
 *	description:	method to execute "read_irqtout"
 *	read trigger timeout
 *
 * @return	timeout
 *
 */
//+------------------------------------------------------------------
Tango::DevUShort MIO::read_irqtout()
{
	Tango::DevUShort	argout ;
	DEBUG_STREAM << "MIO::read_irqtout(): entering... !" << endl;

	//	Add your own code to control device here
	int err;
	mio_t val;
	set_state(Tango::ON);
	if (err = ioctl(mio_dev, MIO_RDIRQTOUT, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_irqtout() error", Tango::ERR);
	}
	argout = val.value;				

	return argout;
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::write_chan_frequency
 *
 *	description:	method to execute "write_chan_frequency"
 *
 * @return void	
 *
 */
//+------------------------------------------------------------------
void MIO::write_chan_frequency(int32_t chan, int32_t val_in)
{
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = (unsigned short) chan;
	val.value = (unsigned short) val_in;
	if (err = ioctl(mio_dev, MIO_WRFREQ, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read a register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::write_freq() error", Tango::ERR);
	}				
}

//+------------------------------------------------------------------
/**
 *	method:	MIO::read_chan_frequency
 *
 *	description:	method to execute "read_chan_set_frequency"
 *
 * @return void	
 *
 */
//+------------------------------------------------------------------
int32_t MIO::read_chan_set_frequency(int32_t chan)
{
	int err;
	mio_t val;
	set_state(Tango::ON);
	val.chan = (unsigned short) chan;
	if (err = ioctl(mio_dev, MIO_RDFREQSET, &val)) {
		set_state(Tango::FAULT);
		TangoSys_MemStream out_stream;
		out_stream << mio_get_errmsg(err) << ends;
		Tango::Except::throw_exception(
			(const char *) "Failed to read frequency register from Multi I/O driver",
			out_stream.str(),
			(const char *) "MIO::read_freq() error", Tango::ERR);
	}

	return (int32_t) val.value;				
		
}



}	//	namespace
